;-------------------------------------------------------------------------
;			automatic baud rate detection
;-------------------------------------------------------------------------
;
;                 ____    __    __ __             ____
;recognize 0x0D:      |__|  |__|  |  |__|__|__|__|
;                     0  1  2  3     5           9
;
.equ	deviation = 20		;tolerance range
.equ	min_pulse = 50		;minimum pulse length (64, 128, 192..)
;
;output: a1:a0 = baud rate divider
;
abaud:
	ldi	timeout, 1 + boottime
	ldi	a0, 0x00
	out	TCCR1A, a0	;normal mode
	ldi	a0, 1<<CS10
	out	TCCR1B, a0	;prescaler 1:1
	ldi	a0, 1<<TOIE1
	out	TIMSK, a0	;enable T1 overflow interrupt
	rcall	meas_low	;synchronize
	rcall	meas_low	;get 1. time
_aba1:  rcall	meas_low	;now measure consecutive
	cpi	a0, min_pulse
	brcc	_aba2
	tst	a1
	breq	_aba1		;time to short
				;check, if new = 4 * last
_aba2:	rcall	div4		;/4
	sub	a2, a0		;compare
	sbc	a3, a1
	brcc	_aba4		;get absolute
	neg	a3
	neg	a2
	sbci	a3, 0
_aba4:	subi	a2, low(deviation)
	sbci	a3, high(deviation)
	brcc	_aba1
	ldi	timeout, 1 + boottime
	subi	a0, 8		;correction
	sbci	a1, 0
;
div16:	rcall	div4		;/16
div4:	rcall	div2
div2:	lsr	a1
	ror	a0
	ret
;-------------------------------------------------------------------------
meas_low:
	ldi	a0, 0
_mel1:	sbic	UART_IN, RXD	;wait until 1->0
	rjmp	_mel1
	out	TCCR1B, a0	;stop T1
	in	a2, TCNT1L
	in	a3, TCNT1H
	out	TCNT1H, a0
	ldi	a0, 7		;stop time = 7 cycle
	out	TCNT1L, a0
	ldi	a0, 1<<CS10
	out	TCCR1B, a0	;run T1
	add	st1l, a2        ;correct software timeout counter
	adc	st1h, a3
	sbci	timeout, 0
	brne	_mel2
	rjmp	userprog	;run user programm
_mel2:	sbis	UART_IN, RXD	;wait until 0->1
	rjmp	_mel2
	in	a0, TCNT1L	;read current low time
	in	a1, TCNT1H
	movw	a3:a2, a5:a4	;get previous low time
	movw	a5:a4, a1:a0	;save current low time
	ret
;-------------------------------------------------------------------------
OVF1int:
	in	savesreg, SREG
	dec	timeout
	brne	_ovf11
	sbrs	flags, f_enable
	rjmp	userprog
_ovf11:	out	SREG, savesreg
	reti
;-------------------------------------------------------------------------
