;-------------------------------------------------------------------------
;				Program Flash / EEPROM
;-------------------------------------------------------------------------
;Input: addrh:addrl = Address
;
FastProg:
	sbr	flags, 1<<f_fast
	rjmp	_pro1
Prog:
	cbr	flags, 1<<f_fast
_pro1:	cbr	flags, 1<<f_write	;mark no write to do
_pro2:	rcall	getchar
	cpi	a0, 0xA5
	brne	_pro3
	rcall	getchar			;get next byte, if A5
	cpi	a0, 0xA5
	brne	_pro4			;End, if != A5
_pro3:	mov	pbyte, a0
	rcall	rdbyte			;read Byte
	sbrs	flags, f_write
	cpse	a0, pbyte		;no program if equal
	rcall	progbyte
	sbrc	flags, f_error
	ret
	adiw	addrh:addrl, 1		;next byte
	mov	a0, addrl
	andi	a0, 0x0F
	sbrc	flags, f_fast
	brne	_pro2			;fast mode: only every 16-th byte
	ldi	a0, key_continue
	rcall	putchar
	rjmp	_pro2

_pro4:	sbrs	flags, f_write
	ret
	rjmp	progpage
;-------------------------------------------------------------------------
PROGBYTE:
	sbrs	flags, f_eeprom
	rjmp	_prb2
	out	EEDR, pbyte
	cli
	sbi	EECR, EEMWE
	sbi	EECR, EEWE
	sei
_prb1:	sbic	EECR, EEWE		;wait until write done
	rjmp	_prb1
	ret
;
_prb2:	cpi	addrh, high( Base * 2 )
	brcs	_prb3			;refuse overwrite itself
	sbr	flags, 1<<f_error
	ret
_prb3:	sbrs	flags, f_write
	rcall	fillbuff		;read page before erase
	mov	yl, addrl
	andi	yl, pagesize * 2 - 1	;byte in page
	ldi	yh, 0
	subi	yl, low ( -page_buff )
	sbci	yh, high( -page_buff )
	st	y+, pbyte
	cpi	yl, page_buff_end	;check page overflow
	breq	progpage
	ret
;-------------------------------------------------------------------------
fillbuff:
	sbr	flags, 1<<f_write	;mark write needed
	ldi	yh, high( page_buff )
	ldi	yl, low( page_buff )
	andi	zl, ~(pagesize * 2 - 1)	;reset Z to page boundary
_fib1:	lpm	a0, z+
	st	y+, a0
	cpi	yl, page_buff_end
	brne	_fib1
	ret
;-------------------------------------------------------------------------
progpage:
	cbr	flags, 1<<f_write	;mark write done
	ldi	yl, low ( page_buff )
	ldi	yh, high( page_buff )
	andi	zl, ~(pagesize * 2 - 1)	;reset Z to page boundary
_prp1:	ld	r0, y+
	ld	r1, y+
	ldi	a0, 1<<SPMEN		;fill page buffer
	rcall	do_spm
	adiw	zh:zl, 2
	cpi	yl, page_buff_end
	brne	_prp1
	subi	zl, low ( pagesize * 2 )
	sbci	zh, high( pagesize * 2 )
	ldi	a0, 1<<SPMEN^1<<PGERS	;erase page
	rcall	do_spm
	ldi	a0, 1<<SPMEN^1<<PGWRT	;write page
	rcall	do_spm
	ldi	a0, 1<<SPMEN^1<<RWWSRE	;reenable RWW
;-------------------------------------------------------------------------
do_spm:	cli
	out_ext	SPMCR, a0
	spm				;do SPM command
	sei
_dos1:	in_ext	a0, SPMCR
	sbrc	a0, SPMEN		;wait until done
	rjmp	_dos1
	ret
;-------------------------------------------------------------------------
